package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.module.base.pojo.enums.GameCategoryEnum;
import com.zanxiang.game.module.manage.pojo.dto.*;
import com.zanxiang.game.module.manage.pojo.params.OrderParam;
import com.zanxiang.game.module.manage.pojo.params.UserOrderListParam;
import com.zanxiang.game.module.manage.pojo.vo.OrderListVO;
import com.zanxiang.game.module.manage.pojo.vo.OrderVO;
import com.zanxiang.game.module.manage.pojo.vo.UserOrderListVO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.mybatis.entity.Agent;
import com.zanxiang.game.module.mybatis.entity.Order;
import com.zanxiang.game.module.mybatis.entity.User;
import com.zanxiang.game.module.mybatis.mapper.OrderMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-07-05
 * @description : 用户订单
 */
@Slf4j
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {

    @Autowired
    private IGameUserRoleService gameUserRoleService;

    @Autowired
    private IGameService gameService;

    @Autowired
    private ICpService cpService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IPayWayService payWayService;

    @Autowired
    private IPayDeviceService payDeviceService;

    @Autowired
    private IAgentService agentService;

    @Override
    public IPage<UserOrderListVO> orderList(UserOrderListParam param) {
        //执行查询
        return page(param.toPage(), new QueryWrapper<Order>().lambda()
                .eq(Order::getUserId, param.getUserId())
                .orderByDesc(Order::getCreateTime)
        ).convert(this::toVo);
    }

    private UserOrderListVO toVo(Order order) {
        if (order == null) {
            return null;
        }
        UserOrderListVO orderListVO = BeanUtil.copy(order, UserOrderListVO.class);
        orderListVO.setId(order.getOrderId());
        //查询角色名称
        GameUserRoleDTO gameUserRoleDTO = gameUserRoleService.getByRoleId(order.getRoleId());
        if (gameUserRoleDTO != null) {
            orderListVO.setRoleName(gameUserRoleDTO.getRoleName());
        }
        //设置游戏名称
        GameDTO gameDTO = gameService.getById(order.getGameId());
        if (gameDTO == null) {
            return orderListVO;
        }
        //设置游戏列表
        orderListVO.setGameCategoryName(GameCategoryEnum.getNameByCategory(gameDTO.getCategory()));
        orderListVO.setGameName(gameDTO.getName());
        return orderListVO;
    }

    @Override
    public OrderListVO orderList(OrderParam param) {
        //渠道列表
        List<AgentDTO> agentDTOList = agentService.listUserAgent(param.getAccountId(), param.getPitcherId(), param.getChannelId());
        //根据条件, 匹配渠道
        if (CollectionUtils.isEmpty(agentDTOList) && !SecurityUtil.isManager()) {
            return new OrderListVO(param.toPage().getSize());
        }
        //渠道id列表
        List<Long> agentIds = agentDTOList.stream().map(AgentDTO::getId).collect(Collectors.toList());
        //管理员可见自然量数据
        if (SecurityUtil.isManager() && Strings.isBlank(param.getAccountId())
                && Strings.isBlank(param.getPitcherId()) && param.getChannelId() == null) {
            agentIds.add(Agent.DEFAULT_AGENT);
        }
        //查询用户id和名字条件
        List<Long> userIds = null;
        if (Strings.isBlank(param.getUserId()) && Strings.isNotBlank(param.getUsername())) {
            userIds = userService.list(new LambdaQueryWrapper<User>()
                    .like(User::getUsername, param.getUsername())
            ).stream().map(User::getId).collect(Collectors.toList());
        }
        //查询订单列表
        IPage<OrderVO> page = page(param.toPage(), getListWrapper(param, agentIds, userIds, "*")).convert(this::orderToVo);
        if (page.getTotal() == 0) {
            return new OrderListVO(param.toPage().getSize());
        }
        //统计：订单金额与实付金额
        Order total = super.getOne(getListWrapper(param, agentIds, userIds, "IFNULL(SUM(amount),0) amount, IFNULL(SUM(real_amount),0) realAmount"));
        HashMap<String, BigDecimal> totalData = new HashMap<>(2);
        if (total == null) {
            totalData.put("totalOrderAmount", new BigDecimal("0.00"));
            totalData.put("totalRealAmount", new BigDecimal("0.00"));
        } else {
            totalData.put("totalOrderAmount", total.getAmount());
            totalData.put("totalRealAmount", total.getRealAmount());
        }
        //分页数据
        List<OrderVO> orderListVOList = page.getRecords();
        //cp信息
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        //玩家信息
        List<Long> userIdList = orderListVOList.stream().map(OrderVO::getUserId).collect(Collectors.toList());
        Map<Long, UserDTO> userMap = userService.getByUserIds(userIdList);
        //渠道
        Map<Long, AgentDTO> agentMap = agentDTOList.stream().collect(Collectors.toMap(AgentDTO::getId, Function.identity()));
        //游戏信息
        Map<Long, GameDTO> gameMap = gameService.gameMap(orderListVOList.stream()
                .map(orderVO -> Arrays.asList(orderVO.getGameId(), orderVO.getRegGameId()))
                .flatMap(List::stream).collect(Collectors.toSet()));
        //支付渠道列表
        Map<Long, PayWayDTO> payWayMap = payWayService.payWayMap();
        //支付方式列表
        Map<Long, PayDeviceDTO> payDeviceMap = payDeviceService.payDeviceMap();
        //字段信息补充更新
        orderListVOList.forEach(vo -> this.update(vo, userMap, gameMap, cpMap,
                payWayMap, payDeviceMap, agentMap));
        //设置返回
        OrderListVO orderListVO = new OrderListVO();
        orderListVO.setSize(page.getSize());
        orderListVO.setTotal(page.getTotal());
        orderListVO.setCurrent(page.getCurrent());
        orderListVO.setPages(page.getPages());
        orderListVO.setData(page.getRecords());
        orderListVO.setTotalData(totalData);
        return orderListVO;
    }

    private void update(OrderVO orderVO, Map<Long, UserDTO> userMap, Map<Long, GameDTO> gameMap, Map<Long, CpDTO> cpMap,
                        Map<Long, PayWayDTO> payWayMap, Map<Long, PayDeviceDTO> payDeviceMap, Map<Long, AgentDTO> agentMap) {
        orderVO.setDateTime(orderVO.getCreateTime().toLocalDate());
        //用户信息补充
        UserDTO userDTO = userMap.get(orderVO.getUserId());
        if (userDTO != null) {
            //注册渠道
            AgentDTO agentDTO = agentMap.get(userDTO.getRegAgentId());
            if (agentDTO != null) {
                orderVO.setRegAgentId(agentDTO.getId());
                orderVO.setRegAgentName(agentDTO.getAgentName());
            }
        }
        //充值渠道
        AgentDTO agentDTO = agentMap.get(orderVO.getAgentId());
        if (agentDTO != null) {
            orderVO.setAccountId(agentDTO.getId());
            orderVO.setPitcherId(agentDTO.getCreateBy());
            orderVO.setPitcherName(agentDTO.getCreateByName());
            orderVO.setAccountType(agentDTO.getAccountType());
        }
        //注册游戏
        GameDTO regGameDTO = gameMap.get(orderVO.getRegGameId());
        if (regGameDTO != null) {
            orderVO.setRegGameName(regGameDTO.getName());
            //注册游戏应用类型
            GameCategoryEnum gameCategoryEnum = GameCategoryEnum.getGameCategory(regGameDTO.getCategory());
            if (gameCategoryEnum != null) {
                orderVO.setRegGameCategoryId(gameCategoryEnum.getId());
                orderVO.setRegGameCategoryName(gameCategoryEnum.getName());
            }
        }
        //CP
        CpDTO cpDTO = cpMap.get(orderVO.getCpId());
        if (cpDTO != null) {
            orderVO.setCpName(cpDTO.getCpName());
        }
        //充值游戏
        GameDTO gameDTO = gameMap.get(orderVO.getGameId());
        if (gameDTO != null) {
            orderVO.setGameName(gameDTO.getName());
            //注册游戏应用类型
            GameCategoryEnum gameCategoryEnum = GameCategoryEnum.getGameCategory(gameDTO.getCategory());
            if (gameCategoryEnum != null) {
                orderVO.setGameCategory(gameCategoryEnum.getName());
                orderVO.setGameCategoryId(gameCategoryEnum.getId());
            }
        }
        //支付渠道
        if (payWayMap.get(orderVO.getPayWayId()) != null) {
            orderVO.setPayWay(payWayMap.get(orderVO.getPayWayId()).getPayWayName());
        }
        //支付场景
        if (payDeviceMap.get(orderVO.getPayDeviceId()) != null) {
            orderVO.setPayDeviceName(payDeviceMap.get(orderVO.getPayDeviceId()).getPayDeviceName());
        }
    }

    private LambdaQueryWrapper<Order> getListWrapper(OrderParam param, List<Long> agentIdList, List<Long> userIdList, String select) {
        return new QueryWrapper<Order>().select(select).lambda()
                .eq(Strings.isNotBlank(param.getOrderId()), Order::getOrderId, param.getOrderId())
                .eq(Strings.isNotBlank(param.getCpOrderId()), Order::getCpOrderId, param.getCpOrderId())
                .eq(Strings.isNotBlank(param.getMerchantOrderNo()), Order::getMerchantOrderNo, param.getMerchantOrderNo())
                .eq(Strings.isNotBlank(param.getUserId()), Order::getUserId, param.getUserId())
                .in(CollectionUtils.isNotEmpty(userIdList), Order::getUserId, userIdList)
                .eq(Strings.isNotBlank(param.getRoleName()), Order::getRoleName, param.getRoleName())
                .ge(param.getRegTimeStart() != null, Order::getRegTime, param.getRegTimeStart() == null ? null : LocalDateTime.of(param.getRegTimeStart(), LocalTime.MIN))
                .le(param.getRegTimeEnd() != null, Order::getRegTime, param.getRegTimeEnd() == null ? null : LocalDateTime.of(param.getRegTimeEnd(), LocalTime.MAX))
                .eq(param.getCpId() != null, Order::getCpId, param.getCpId())
                .eq(param.getGameId() != null, Order::getGameId, param.getGameId())
                .in(CollectionUtils.isNotEmpty(agentIdList), Order::getAgentId, agentIdList)
                .eq(Strings.isNotBlank(param.getDeviceSystem()), Order::getDeviceSystem, param.getDeviceSystem())
                .eq(param.getIsSwitch() != null, Order::getIsSwitch, param.getIsSwitch())
                .eq(Strings.isNotBlank(param.getProductName()), Order::getProductName, param.getProductName())
                .eq(Strings.isNotBlank(param.getPayWayId()), Order::getPayWayId, param.getPayWayId())
                .eq(Strings.isNotBlank(param.getMerchantNo()), Order::getMerchantNo, param.getMerchantNo())
                .eq(param.getStatus() != null, Order::getStatus, param.getStatus())
                .eq(param.getCpStatus() != null, Order::getCpStatus, param.getCpStatus())
                .eq(param.getIsFirstRecharge() != null, Order::getIsFirstRecharge, param.getIsFirstRecharge())
                .eq(param.getCallBackStatus() != null, Order::getCallBackStatus, param.getCallBackStatus())
                .ge(param.getBeginDate() != null, Order::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, Order::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .orderByDesc(Order::getCreateTime);
    }


    private OrderVO orderToVo(Order order) {
        if (Objects.isNull(order)) {
            return null;
        }
        return BeanUtil.copy(order, OrderVO.class);
    }
}
